@@ -25,7 +25,7 @@ which automatically terminates the run at the end of the ``with`` block.
 The fluent tracking API is not currently threadsafe. Any concurrent callers to the tracking API must
 implement mutual exclusion manually.
 
-For a lower level API, see the :py:mod:`mlflow.tracking` module.
+For a lower level API, see the :py:mod:`mlflow.client` module.
 """
 from mlflow.version import VERSION as __version__  # pylint: disable=unused-import
 from mlflow.utils.logging_utils import _configure_mlflow_loggers
@@ -37,35 +37,42 @@ import mlflow.tracking.fluent
 # See: https://github.com/numpy/numpy/pull/432/commits/170ed4e33d6196d7
 import warnings
 
-warnings.filterwarnings("ignore", message="numpy.dtype size changed")  # noqa: E402
-warnings.filterwarnings("ignore", message="numpy.ufunc size changed")  # noqa: E402
+warnings.filterwarnings("ignore", message="numpy.dtype size changed")
+warnings.filterwarnings("ignore", message="numpy.ufunc size changed")
 
-import mlflow.projects as projects  # noqa: E402
-import mlflow.tracking as tracking  # noqa: E402
+from mlflow import projects
+from mlflow import tracking
+import mlflow.models
+import mlflow.artifacts
+import mlflow.pipelines
+import mlflow.client
 
 # model flavors
 _model_flavors_supported = []
 try:
     # pylint: disable=unused-import
-    import mlflow.catboost as catboost  # noqa: E402
-    import mlflow.fastai as fastai  # noqa: E402
-    import mlflow.gluon as gluon  # noqa: E402
-    import mlflow.h2o as h2o  # noqa: E402
-    import mlflow.keras as keras  # noqa: E402
-    import mlflow.lightgbm as lightgbm  # noqa: E402
-    import mlflow.mleap as mleap  # noqa: E402
-    import mlflow.onnx as onnx  # noqa: E402
-    import mlflow.pyfunc as pyfunc  # noqa: E402
-    import mlflow.pytorch as pytorch  # noqa: E402
-    import mlflow.sklearn as sklearn  # noqa: E402
-    import mlflow.spacy as spacy  # noqa: E402
-    import mlflow.spark as spark  # noqa: E402
-    import mlflow.statsmodels as statsmodels  # noqa: E402
-    import mlflow.tensorflow as tensorflow  # noqa: E402
-    import mlflow.xgboost as xgboost  # noqa: E402
-    import mlflow.shap as shap  # noqa: E402
-    import mlflow.pyspark as pyspark  # noqa: E402
-    import mlflow.paddle as paddle
+    from mlflow import catboost
+    from mlflow import fastai
+    from mlflow import gluon
+    from mlflow import h2o
+    from mlflow import keras
+    from mlflow import lightgbm
+    from mlflow import mleap
+    from mlflow import onnx
+    from mlflow import pyfunc
+    from mlflow import pytorch
+    from mlflow import sklearn
+    from mlflow import spacy
+    from mlflow import spark
+    from mlflow import statsmodels
+    from mlflow import tensorflow
+    from mlflow import xgboost
+    from mlflow import shap
+    from mlflow import pyspark
+    from mlflow import paddle
+    from mlflow import prophet
+    from mlflow import pmdarima
+    from mlflow import diviner
 
     _model_flavors_supported = [
         "catboost",
@@ -86,6 +93,9 @@ try:
         "xgboost",
         "shap",
         "paddle",
+        "prophet",
+        "pmdarima",
+        "diviner",
     ]
 except ImportError as e:
     # We are conditional loading these commands since the skinny client does
@@ -95,9 +105,9 @@ except ImportError as e:
 
 _configure_mlflow_loggers(root_module_name=__name__)
 
-# TODO: Uncomment this block when deprecating Python 3.6 support
+# TODO: Comment out this block when we deprecate support for python 3.7.
 # _major = 3
-# _minor = 6
+# _minor = 7
 # _deprecated_version = (_major, _minor)
 # _min_supported_version = (_major, _minor + 1)
 
@@ -138,18 +148,24 @@ set_registry_uri = tracking.set_registry_uri
 get_experiment = mlflow.tracking.fluent.get_experiment
 get_experiment_by_name = mlflow.tracking.fluent.get_experiment_by_name
 list_experiments = mlflow.tracking.fluent.list_experiments
+search_experiments = mlflow.tracking.fluent.search_experiments
 get_tracking_uri = tracking.get_tracking_uri
 get_registry_uri = tracking.get_registry_uri
+is_tracking_uri_set = tracking.is_tracking_uri_set
 create_experiment = mlflow.tracking.fluent.create_experiment
 set_experiment = mlflow.tracking.fluent.set_experiment
 log_params = mlflow.tracking.fluent.log_params
 log_metrics = mlflow.tracking.fluent.log_metrics
+set_experiment_tags = mlflow.tracking.fluent.set_experiment_tags
+set_experiment_tag = mlflow.tracking.fluent.set_experiment_tag
 set_tags = mlflow.tracking.fluent.set_tags
 delete_experiment = mlflow.tracking.fluent.delete_experiment
 delete_run = mlflow.tracking.fluent.delete_run
 register_model = mlflow.tracking._model_registry.fluent.register_model
 autolog = mlflow.tracking.fluent.autolog
-
+evaluate = mlflow.models.evaluate
+last_active_run = mlflow.tracking.fluent.last_active_run
+MlflowClient = mlflow.client.MlflowClient
 
 run = projects.run
 
@@ -159,6 +175,8 @@ __all__ = [
     "log_params",
     "log_metric",
     "log_metrics",
+    "set_experiment_tags",
+    "set_experiment_tag",
     "set_tag",
     "set_tags",
     "delete_tag",
@@ -175,9 +193,11 @@ __all__ = [
     "get_artifact_uri",
     "get_tracking_uri",
     "set_tracking_uri",
+    "is_tracking_uri_set",
     "get_experiment",
     "get_experiment_by_name",
     "list_experiments",
+    "search_experiments",
     "create_experiment",
     "set_experiment",
     "delete_experiment",
@@ -189,4 +209,7 @@ __all__ = [
     "set_registry_uri",
     "list_run_infos",
     "autolog",
+    "evaluate",
+    "last_active_run",
+    "MlflowClient",
 ] + _model_flavors_supported
