@@ -1,16 +1,19 @@
 import json
 import os
+import re
 import sys
 import logging
 
 import click
 from click import UsageError
+from datetime import timedelta
 
 import mlflow.db
 import mlflow.experiments
 import mlflow.deployments.cli
 import mlflow.pipelines.cli
-import mlflow.projects as projects
+from mlflow import projects
+from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
 import mlflow.runs
 import mlflow.store.artifact.cli
 from mlflow import version
@@ -249,6 +252,14 @@ def _validate_server_args(gunicorn_opts=None, workers=None, waitress_opts=None):
     "(e.g. 'file:///absolute/path/to/directory'). By default, data will be logged "
     f"to {DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH}",
 )
+@click.option(
+    "--registry-store-uri",
+    metavar="URI",
+    default=None,
+    help="URI to which to persist registered models. Acceptable URIs are "
+    "SQLAlchemy-compatible database connection strings (e.g. 'sqlite:///path/to/file.db'). "
+    "If not specified, `backend-store-uri` is used.",
+)
 @click.option(
     "--default-artifact-root",
     metavar="URI",
@@ -266,7 +277,13 @@ def _validate_server_args(gunicorn_opts=None, workers=None, waitress_opts=None):
 @cli_args.PORT
 @cli_args.HOST
 def ui(
-    backend_store_uri, default_artifact_root, serve_artifacts, artifacts_destination, port, host
+    backend_store_uri,
+    registry_store_uri,
+    default_artifact_root,
+    serve_artifacts,
+    artifacts_destination,
+    port,
+    host,
 ):
     """
     Launch the MLflow tracking UI for local viewing of run results. To launch a production
@@ -284,12 +301,16 @@ def ui(
     if not backend_store_uri:
         backend_store_uri = DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH
 
+    # the default setting of registry_store_uri is same as backend_store_uri
+    if not registry_store_uri:
+        registry_store_uri = backend_store_uri
+
     default_artifact_root = resolve_default_artifact_root(
         serve_artifacts, default_artifact_root, backend_store_uri, resolve_to_local=True
     )
 
     try:
-        initialize_backend_stores(backend_store_uri, default_artifact_root)
+        initialize_backend_stores(backend_store_uri, registry_store_uri, default_artifact_root)
     except Exception as e:
         _logger.error("Error initializing backend store")
         _logger.exception(e)
@@ -299,6 +320,7 @@ def ui(
     try:
         _run_server(
             backend_store_uri,
+            registry_store_uri,
             default_artifact_root,
             serve_artifacts,
             False,
@@ -338,6 +360,14 @@ def _validate_static_prefix(ctx, param, value):  # pylint: disable=unused-argume
     "(e.g. 'file:///absolute/path/to/directory'). By default, data will be logged "
     "to the ./mlruns directory.",
 )
+@click.option(
+    "--registry-store-uri",
+    metavar="URI",
+    default=None,
+    help="URI to which to persist registered models. Acceptable URIs are "
+    "SQLAlchemy-compatible database connection strings (e.g. 'sqlite:///path/to/file.db'). "
+    "If not specified, `backend-store-uri` is used.",
+)
 @click.option(
     "--default-artifact-root",
     metavar="URI",
@@ -388,6 +418,7 @@ def _validate_static_prefix(ctx, param, value):  # pylint: disable=unused-argume
 )
 def server(
     backend_store_uri,
+    registry_store_uri,
     default_artifact_root,
     serve_artifacts,
     artifacts_only,
@@ -417,13 +448,17 @@ def server(
     if not backend_store_uri:
         backend_store_uri = DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH
 
+    # the default setting of registry_store_uri is same as backend_store_uri
+    if not registry_store_uri:
+        registry_store_uri = backend_store_uri
+
     default_artifact_root = resolve_default_artifact_root(
         serve_artifacts, default_artifact_root, backend_store_uri
     )
     artifacts_only_config_validation(artifacts_only, backend_store_uri)
 
     try:
-        initialize_backend_stores(backend_store_uri, default_artifact_root)
+        initialize_backend_stores(backend_store_uri, registry_store_uri, default_artifact_root)
     except Exception as e:
         _logger.error("Error initializing backend store")
         _logger.exception(e)
@@ -432,6 +467,7 @@ def server(
     try:
         _run_server(
             backend_store_uri,
+            registry_store_uri,
             default_artifact_root,
             serve_artifacts,
             artifacts_only,
@@ -450,6 +486,13 @@ def server(
 
 
 @cli.command(short_help="Permanently delete runs in the `deleted` lifecycle stage.")
+@click.option(
+    "--older-than",
+    default=None,
+    help="Optional. Remove run(s) older than the specified time limit. "
+    "Specify a string in #d#h#m#s format. Float values are also supported."
+    "For example: --older-than 1d2h3m4s, --older-than 1.2d3h4m5s",
+)
 @click.option(
     "--backend-store-uri",
     metavar="PATH",
@@ -467,7 +510,7 @@ def server(
     " are not specified, data is removed for all runs in the `deleted`"
     " lifecycle stage.",
 )
-def gc(backend_store_uri, run_ids):
+def gc(older_than, backend_store_uri, run_ids):
     """
     Permanently delete runs in the `deleted` lifecycle stage from the specified backend store.
     This command deletes all artifacts and metadata associated with the specified runs.
@@ -477,8 +520,28 @@ def gc(backend_store_uri, run_ids):
         raise MlflowException(
             "This cli can only be used with a backend that allows hard-deleting runs"
         )
+
+    time_delta = 0
+
+    if older_than is not None:
+        regex = re.compile(
+            r"^((?P<days>[\.\d]+?)d)?((?P<hours>[\.\d]+?)h)?((?P<minutes>[\.\d]+?)m)"
+            r"?((?P<seconds>[\.\d]+?)s)?$"
+        )
+        parts = regex.match(older_than)
+        if parts is None:
+            raise MlflowException(
+                "Could not parse any time information from '{}'. "
+                "Examples of valid strings: '8h', '2d8h5m20s', '2m4s'".format(older_than),
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+        time_params = {name: float(param) for name, param in parts.groupdict().items() if param}
+        time_delta = int(timedelta(**time_params).total_seconds() * 1000)
+
+    deleted_run_ids_older_than = backend_store._get_deleted_runs(older_than=time_delta)
     if not run_ids:
-        run_ids = backend_store._get_deleted_runs()
+        run_ids = deleted_run_ids_older_than
+
     else:
         run_ids = run_ids.split(",")
 
@@ -489,6 +552,13 @@ def gc(backend_store_uri, run_ids):
                 "Run % is not in `deleted` lifecycle stage. Only runs in "
                 "`deleted` lifecycle stage can be deleted." % run_id
             )
+        # raise MlflowException if run_id is newer than older_than parameter
+        if older_than and run_id not in deleted_run_ids_older_than:
+            raise MlflowException(
+                f"Run {run_id} is not older than the required age. "
+                f"Only runs older than {older_than} can be deleted.",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
         artifact_repo = get_artifact_repository(run.info.artifact_uri)
         artifact_repo.delete_artifacts()
         backend_store._hard_delete_run(run_id)
@@ -502,19 +572,30 @@ cli.add_command(mlflow.runs.commands)
 cli.add_command(mlflow.db.commands)
 cli.add_command(mlflow.pipelines.cli.commands)
 
+# We are conditional loading these commands since the skinny client does
+# not support them due to the pandas and numpy dependencies of MLflow Models
 try:
-    # pylint: disable=unused-import
-    import mlflow.models.cli
-    import mlflow.azureml.cli
-    import mlflow.sagemaker.cli
+    import mlflow.models.cli  # pylint: disable=unused-import
 
-    cli.add_command(mlflow.azureml.cli.commands)
-    cli.add_command(mlflow.sagemaker.cli.commands)
     cli.add_command(mlflow.models.cli.commands)
 except ImportError as e:
-    # We are conditional loading these commands since the skinny client does
-    # not support them due to the pandas and numpy dependencies of MLflow Models
     pass
 
+
+try:
+    import mlflow.azureml.cli  # pylint: disable=unused-import
+
+    cli.add_command(mlflow.azureml.cli.commands)
+except ImportError as e:
+    pass
+
+try:
+    import mlflow.sagemaker.cli  # pylint: disable=unused-import
+
+    cli.add_command(mlflow.sagemaker.cli.commands)
+except ImportError as e:
+    pass
+
+
 if __name__ == "__main__":
     cli()
