@@ -1,6 +1,5 @@
 import logging
 import click
-import os
 
 from mlflow.models import Model
 from mlflow.models.model import MLMODEL_FILE_NAME
@@ -9,6 +8,7 @@ from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.utils import cli_args
 from mlflow.utils.file_utils import TempDir
 from mlflow.utils.uri import append_to_uri_path
+from mlflow.utils import env_manager as _EnvManager
 
 _logger = logging.getLogger(__name__)
 
@@ -28,10 +28,23 @@ def commands():
 @cli_args.MODEL_URI
 @cli_args.PORT
 @cli_args.HOST
+@cli_args.TIMEOUT
 @cli_args.WORKERS
 @cli_args.NO_CONDA
+@cli_args.ENV_MANAGER
 @cli_args.INSTALL_MLFLOW
-def serve(model_uri, port, host, workers, no_conda=False, install_mlflow=False):
+@cli_args.ENABLE_MLSERVER
+def serve(
+    model_uri,
+    port,
+    host,
+    timeout,
+    workers,
+    no_conda,  # pylint: disable=unused-argument
+    env_manager=None,
+    install_mlflow=False,
+    enable_mlserver=False,
+):
     """
     Serve a model saved with MLflow by launching a webserver on the specified host and port.
     The command supports models with the ``python_function`` or ``crate`` (R Function) flavor.
@@ -51,9 +64,12 @@ def serve(model_uri, port, host, workers, no_conda=False, install_mlflow=False):
             "data": [[1, 2, 3], [4, 5, 6]]
         }'
     """
+    env_manager = env_manager or _EnvManager.CONDA
     return _get_flavor_backend(
-        model_uri, no_conda=no_conda, workers=workers, install_mlflow=install_mlflow
-    ).serve(model_uri=model_uri, port=port, host=host)
+        model_uri, env_manager=env_manager, workers=workers, install_mlflow=install_mlflow
+    ).serve(
+        model_uri=model_uri, port=port, host=host, timeout=timeout, enable_mlserver=enable_mlserver
+    )
 
 
 @commands.command("predict")
@@ -86,18 +102,29 @@ def serve(model_uri, port, host, workers, no_conda=False, install_mlflow=False):
     ".html",
 )
 @cli_args.NO_CONDA
+@cli_args.ENV_MANAGER
 @cli_args.INSTALL_MLFLOW
 def predict(
-    model_uri, input_path, output_path, content_type, json_format, no_conda, install_mlflow
+    model_uri,
+    input_path,
+    output_path,
+    content_type,
+    json_format,
+    no_conda,  # pylint: disable=unused-argument
+    env_manager,
+    install_mlflow,
 ):
     """
     Generate predictions in json format using a saved MLflow model. For information about the input
     data formats accepted by this function, see the following documentation:
     https://www.mlflow.org/docs/latest/models.html#built-in-deployment-tools.
     """
+    env_manager = env_manager or _EnvManager.CONDA
     if content_type == "json" and json_format not in ("split", "records"):
         raise Exception("Unsupported json format '{}'.".format(json_format))
-    return _get_flavor_backend(model_uri, no_conda=no_conda, install_mlflow=install_mlflow).predict(
+    return _get_flavor_backend(
+        model_uri, env_manager=env_manager, install_mlflow=install_mlflow
+    ).predict(
         model_uri=model_uri,
         input_path=input_path,
         output_path=output_path,
@@ -109,42 +136,64 @@ def predict(
 @commands.command("prepare-env")
 @cli_args.MODEL_URI
 @cli_args.NO_CONDA
+@cli_args.ENV_MANAGER
 @cli_args.INSTALL_MLFLOW
-def prepare_env(model_uri, no_conda, install_mlflow):
+def prepare_env(
+    model_uri,
+    no_conda,  # pylint: disable=unused-argument
+    env_manager,
+    install_mlflow,
+):
     """
-    **EXPERIMENTAL**: Performs any preparation necessary to predict or serve the model, for example
+    Performs any preparation necessary to predict or serve the model, for example
     downloading dependencies or initializing a conda environment. After preparation,
     calling predict or serve should be fast.
-
-    This method is experimental and may be removed in a future release without warning.
     """
+    env_manager = env_manager or _EnvManager.CONDA
     return _get_flavor_backend(
-        model_uri, no_conda=no_conda, install_mlflow=install_mlflow
+        model_uri, env_manager=env_manager, install_mlflow=install_mlflow
     ).prepare_env(model_uri=model_uri)
 
 
 @commands.command("build-docker")
-@cli_args.MODEL_URI
+@cli_args.MODEL_URI_BUILD_DOCKER
 @click.option("--name", "-n", default="mlflow-pyfunc-servable", help="Name to use for built image")
+@cli_args.ENV_MANAGER
+@cli_args.MLFLOW_HOME
 @cli_args.INSTALL_MLFLOW
-def build_docker(model_uri, name, install_mlflow):
+@cli_args.ENABLE_MLSERVER
+def build_docker(model_uri, name, env_manager, mlflow_home, install_mlflow, enable_mlserver):
     """
-    **EXPERIMENTAL**: Builds a Docker image whose default entrypoint serves the specified MLflow
-    model at port 8080 within the container, using the 'python_function' flavor.
+    Builds a Docker image whose default entrypoint serves an MLflow model at port 8080, using the
+    python_function flavor. The container serves the model referenced by ``--model-uri``, if
+    specified when ``build-docker`` is called. If ``--model-uri`` is not specified when build_docker
+    is called, an MLflow Model directory must be mounted as a volume into the /opt/ml/model
+    directory in the container.
 
-    For example, the following command builds a docker image named 'my-image-name' that serves the
-    model from run 'some-run-uuid' at run-relative artifact path 'my-model':
-
-    .. code:: bash
-
-        mlflow models build-docker -m "runs:/some-run-uuid/my-model" -n "my-image-name"
-
-    We can then serve the model, exposing it at port 5001 on the host via:
+    Building a Docker image with ``--model-uri``:
 
     .. code:: bash
 
+        # Build a Docker image named 'my-image-name' that serves the model from run 'some-run-uuid'
+        # at run-relative artifact path 'my-model'
+        mlflow models build-docker --model-uri "runs:/some-run-uuid/my-model" --name "my-image-name"
+        # Serve the model
         docker run -p 5001:8080 "my-image-name"
 
+    Building a Docker image without ``--model-uri``:
+
+    .. code:: bash
+
+        # Build a generic Docker image named 'my-image-name'
+        mlflow models build-docker --name "my-image-name"
+        # Mount the model stored in '/local/path/to/artifacts/model' and serve it
+        docker run --rm -p 5001:8080 -v /local/path/to/artifacts/model:/opt/ml/model "my-image-name"
+
+    .. warning::
+
+        The image built without ``--model-uri`` doesn't support serving models with RFunc / Java
+        MLeap model server.
+
     NB: by default, the container will start nginx and gunicorn processes. If you don't need the
     nginx process to be started (for instance if you deploy your container to Google Cloud Run),
     you can disable it via the DISABLE_NGINX environment variable:
@@ -153,32 +202,34 @@ def build_docker(model_uri, name, install_mlflow):
 
         docker run -p 5001:8080 -e DISABLE_NGINX=true "my-image-name"
 
-
     See https://www.mlflow.org/docs/latest/python_api/mlflow.pyfunc.html for more information on the
     'python_function' flavor.
-
-    This command is experimental (may be changed or removed in a future release without warning)
-    and does not guarantee that the arguments nor format of the Docker container will remain the
-    same.
     """
-    mlflow_home = os.environ.get("MLFLOW_HOME", None)
-    _get_flavor_backend(model_uri, docker_build=True).build_image(
-        model_uri, name, mlflow_home=mlflow_home, install_mlflow=install_mlflow
+    env_manager = env_manager or _EnvManager.CONDA
+    _get_flavor_backend(model_uri, docker_build=True, env_manager=env_manager).build_image(
+        model_uri,
+        name,
+        mlflow_home=mlflow_home,
+        install_mlflow=install_mlflow,
+        enable_mlserver=enable_mlserver,
     )
 
 
 def _get_flavor_backend(model_uri, **kwargs):
     from mlflow.models.flavor_backend_registry import get_flavor_backend
 
-    with TempDir() as tmp:
-        if ModelsArtifactRepository.is_models_uri(model_uri):
-            underlying_model_uri = ModelsArtifactRepository.get_underlying_uri(model_uri)
-        else:
-            underlying_model_uri = model_uri
-        local_path = _download_artifact_from_uri(
-            append_to_uri_path(underlying_model_uri, MLMODEL_FILE_NAME), output_path=tmp.path()
-        )
-        model = Model.load(local_path)
+    if model_uri:
+        with TempDir() as tmp:
+            if ModelsArtifactRepository.is_models_uri(model_uri):
+                underlying_model_uri = ModelsArtifactRepository.get_underlying_uri(model_uri)
+            else:
+                underlying_model_uri = model_uri
+            local_path = _download_artifact_from_uri(
+                append_to_uri_path(underlying_model_uri, MLMODEL_FILE_NAME), output_path=tmp.path()
+            )
+            model = Model.load(local_path)
+    else:
+        model = None
     flavor_name, flavor_backend = get_flavor_backend(model, **kwargs)
     if flavor_backend is None:
         raise Exception("No suitable flavor backend was found for the model.")
