@@ -10,14 +10,16 @@ import sqlalchemy
 
 from mlflow.exceptions import MlflowException
 from mlflow.store.tracking.dbmodels.initial_models import Base as InitialBase
-from mlflow.protos.databricks_pb2 import INTERNAL_ERROR
+from mlflow.protos.databricks_pb2 import BAD_REQUEST, INTERNAL_ERROR
 from mlflow.store.db.db_types import SQLITE
+from mlflow.environment_variables import (
+    MLFLOW_SQLALCHEMYSTORE_POOL_SIZE,
+    MLFLOW_SQLALCHEMYSTORE_POOL_RECYCLE,
+    MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW,
+)
 
 _logger = logging.getLogger(__name__)
 
-
-MLFLOW_SQLALCHEMYSTORE_POOL_SIZE = "MLFLOW_SQLALCHEMYSTORE_POOL_SIZE"
-MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW = "MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW"
 MAX_RETRY_COUNT = 15
 
 
@@ -82,6 +84,9 @@ def _get_managed_session_maker(SessionMaker, db_type):
         except MlflowException:
             session.rollback()
             raise
+        except sqlalchemy.exc.SQLAlchemyError as e:
+            session.rollback()
+            raise MlflowException(message=e, error_code=BAD_REQUEST)
         except Exception as e:
             session.rollback()
             raise MlflowException(message=e, error_code=INTERNAL_ERROR)
@@ -152,40 +157,6 @@ def _get_schema_version(engine):
         return mc.get_current_revision()
 
 
-def _is_initialized_before_mlflow_1(engine):
-    """
-    Returns true if the database at the specified URL was initialized before MLflow 1.0, False
-    otherwise.
-    A database is initialized before MLflow 1.0 if and only if its revision ID is set to None.
-    """
-    return _get_schema_version(engine) is None
-
-
-def _upgrade_db_initialized_before_mlflow_1(engine):
-    """
-    Upgrades the schema of an MLflow tracking database created prior to MLflow 1.0, removing
-    duplicate constraint names. This method performs a one-time update for pre-1.0 users that we
-    plan to make available in MLflow 1.0 but remove in successive versions (e.g. MLflow 1.1),
-    after which we will assume that effectively all databases have been initialized using the schema
-    in mlflow.store.dbmodels.initial_models (with a small number of special-case databases
-    initialized pre-1.0 and migrated to have the same schema as mlflow.store.dbmodels.initial_models
-    via this method).
-    TODO: remove this method in MLflow 1.1.
-    """
-    # alembic adds significant import time, so we import it lazily
-    from alembic import command
-
-    _logger.info("Updating database tables in preparation for MLflow 1.0 schema migrations ")
-    alembic_dir = os.path.join(_get_package_dir(), "temporary_db_migrations_for_pre_1_users")
-    config = _get_alembic_config(str(engine.url), alembic_dir)
-    command.upgrade(config, "heads")
-    # Reset the alembic version to "base" (the 'first' version) so that a) the versioning system
-    # is unaware that this migration occurred and b) subsequent migrations, like the migration to
-    # add metric steps, do not need to depend on this one. This allows us to eventually remove this
-    # method and the associated migration e.g. in MLflow 1.1.
-    command.stamp(config, "base")
-
-
 def create_sqlalchemy_engine_with_retry(db_uri):
     attempts = 0
     while True:
@@ -196,7 +167,7 @@ def create_sqlalchemy_engine_with_retry(db_uri):
             return engine
         except Exception as e:
             if attempts < MAX_RETRY_COUNT:
-                sleep_duration = 0.1 * ((2 ** attempts) - 1)
+                sleep_duration = 0.1 * ((2**attempts) - 1)
                 _logger.warning(
                     "SQLAlchemy engine could not be created. The following exception is caught.\n"
                     "%s\nOperation will be retried in %.1f seconds",
@@ -209,8 +180,9 @@ def create_sqlalchemy_engine_with_retry(db_uri):
 
 
 def create_sqlalchemy_engine(db_uri):
-    pool_size = os.environ.get(MLFLOW_SQLALCHEMYSTORE_POOL_SIZE)
-    pool_max_overflow = os.environ.get(MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW)
+    pool_size = MLFLOW_SQLALCHEMYSTORE_POOL_SIZE.get()
+    pool_max_overflow = MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW.get()
+    pool_recycle = MLFLOW_SQLALCHEMYSTORE_POOL_RECYCLE.get()
     pool_kwargs = {}
     # Send argument only if they have been injected.
     # Some engine does not support them (for example sqllite)
@@ -218,6 +190,8 @@ def create_sqlalchemy_engine(db_uri):
         pool_kwargs["pool_size"] = int(pool_size)
     if pool_max_overflow:
         pool_kwargs["max_overflow"] = int(pool_max_overflow)
+    if pool_recycle:
+        pool_kwargs["pool_recycle"] = int(pool_recycle)
     if pool_kwargs:
         _logger.info("Create SQLAlchemy engine with pool options %s", pool_kwargs)
     return sqlalchemy.create_engine(db_uri, pool_pre_ping=True, **pool_kwargs)
