@@ -8,6 +8,7 @@ from mlflow.protos.service_pb2 import (
     GetRun,
     SearchRuns,
     ListExperiments,
+    SearchExperiments,
     GetMetricHistory,
     LogMetric,
     LogParam,
@@ -26,6 +27,7 @@ from mlflow.protos.service_pb2 import (
     GetExperimentByName,
 )
 from mlflow.store.tracking.abstract_store import AbstractStore
+from mlflow.store.entities.paged_list import PagedList
 from mlflow.utils.proto_json_utils import message_to_json
 from mlflow.utils.rest_utils import (
     call_endpoint,
@@ -54,18 +56,59 @@ class RestStore(AbstractStore):
         response_proto = api.Response()
         return call_endpoint(self.get_host_creds(), endpoint, method, json_body, response_proto)
 
-    def list_experiments(self, view_type=ViewType.ACTIVE_ONLY):
+    def list_experiments(
+        self,
+        view_type=ViewType.ACTIVE_ONLY,
+        max_results=None,
+        page_token=None,
+    ):
         """
-        :return: a list of all known Experiment objects
+        :param view_type: Qualify requested type of experiments.
+        :param max_results: If passed, specifies the maximum number of experiments desired. If not
+                            passed, the server will pick a maximum number of results to return.
+        :param page_token: Token specifying the next page of results. It should be obtained from
+                            a ``list_experiments`` call.
+        :return: A :py:class:`PagedList <mlflow.store.entities.PagedList>` of
+                 :py:class:`Experiment <mlflow.entities.Experiment>` objects. The pagination token
+                 for the next page can be obtained via the ``token`` attribute of the object.
         """
-        req_body = message_to_json(ListExperiments(view_type=view_type))
+        req_body = message_to_json(
+            ListExperiments(view_type=view_type, max_results=max_results, page_token=page_token)
+        )
         response_proto = self._call_endpoint(ListExperiments, req_body)
-        return [
-            Experiment.from_proto(experiment_proto)
-            for experiment_proto in response_proto.experiments
-        ]
+        experiments = [Experiment.from_proto(x) for x in response_proto.experiments]
+        # If the response doesn't contain `next_page_token`, `response_proto.next_page_token`
+        # returns an empty string (default value for a string proto field).
+        token = (
+            response_proto.next_page_token if response_proto.HasField("next_page_token") else None
+        )
+        return PagedList(experiments, token)
 
-    def create_experiment(self, name, artifact_location=None):
+    def search_experiments(
+        self,
+        view_type=ViewType.ACTIVE_ONLY,
+        max_results=None,
+        filter_string=None,
+        order_by=None,
+        page_token=None,
+    ):
+        req_body = message_to_json(
+            SearchExperiments(
+                view_type=view_type,
+                max_results=max_results,
+                page_token=page_token,
+                order_by=order_by,
+                filter=filter_string,
+            )
+        )
+        response_proto = self._call_endpoint(SearchExperiments, req_body)
+        experiments = [Experiment.from_proto(x) for x in response_proto.experiments]
+        token = (
+            response_proto.next_page_token if response_proto.HasField("next_page_token") else None
+        )
+        return PagedList(experiments, token)
+
+    def create_experiment(self, name, artifact_location=None, tags=None):
         """
         Create a new experiment.
         If an experiment with the given name already exists, throws exception.
@@ -74,7 +117,10 @@ class RestStore(AbstractStore):
 
         :return: experiment_id (string) for the newly created experiment if successful, else None
         """
-        req_body = message_to_json(CreateExperiment(name=name, artifact_location=artifact_location))
+        tag_protos = [tag.to_proto() for tag in tags] if tags else []
+        req_body = message_to_json(
+            CreateExperiment(name=name, artifact_location=artifact_location, tags=tag_protos)
+        )
         response_proto = self._call_endpoint(CreateExperiment, req_body)
         return response_proto.experiment_id
 
@@ -118,7 +164,7 @@ class RestStore(AbstractStore):
         return Run.from_proto(response_proto.run)
 
     def update_run_info(self, run_id, run_status, end_time):
-        """ Updates the metadata of the specified run. """
+        """Updates the metadata of the specified run."""
         req_body = message_to_json(
             UpdateRun(run_uuid=run_id, run_id=run_id, status=run_status, end_time=end_time)
         )
@@ -132,7 +178,8 @@ class RestStore(AbstractStore):
 
         :param experiment_id: ID of the experiment for this run
         :param user_id: ID of the user launching this run
-        :param source_type: Enum (integer) describing the source of the run
+        :param start_time: timestamp of the initialization of the run
+        :param tags: tags to apply to this run at initialization
 
         :return: The created Run object
         """
@@ -267,15 +314,13 @@ class RestStore(AbstractStore):
                 databricks_pb2.RESOURCE_DOES_NOT_EXIST
             ):
                 return None
-            elif e.error_code == databricks_pb2.ErrorCode.Name(
-                databricks_pb2.REQUEST_LIMIT_EXCEEDED
-            ):
-                raise e
-            # Fall back to using ListExperiments-based implementation.
-            for experiment in self.list_experiments(ViewType.ALL):
-                if experiment.name == experiment_name:
-                    return experiment
-            return None
+            elif e.error_code == databricks_pb2.ErrorCode.Name(databricks_pb2.ENDPOINT_NOT_FOUND):
+                # Fall back to using ListExperiments-based implementation.
+                for experiment in self.list_experiments(ViewType.ALL):
+                    if experiment.name == experiment_name:
+                        return experiment
+                return None
+            raise e
 
     def log_batch(self, run_id, metrics, params, tags):
         metric_protos = [metric.to_proto() for metric in metrics]
@@ -312,8 +357,19 @@ class DatabricksRestStore(RestStore):
                 return None
             elif e.error_code == databricks_pb2.ErrorCode.Name(databricks_pb2.ENDPOINT_NOT_FOUND):
                 # Fall back to using ListExperiments-based implementation.
-                for experiment in self.list_experiments(ViewType.ALL):
-                    if experiment.name == experiment_name:
-                        return experiment
+                for experiments in self._paginate_list_experiments(ViewType.ALL):
+                    for experiment in experiments:
+                        if experiment.name == experiment_name:
+                            return experiment
                 return None
             raise e
+
+    def _paginate_list_experiments(self, view_type):
+        page_token = None
+        while True:
+            experiments = self.list_experiments(view_type=view_type, page_token=page_token)
+            yield experiments
+
+            if not experiments.token:
+                break
+            page_token = experiments.token
