@@ -19,7 +19,7 @@ _logger = logging.getLogger(__name__)
 AWAIT_MODEL_VERSION_CREATE_SLEEP_DURATION_SECONDS = 3
 
 
-class ModelRegistryClient(object):
+class ModelRegistryClient:
     """
     Client of an MLflow Model Registry Server that creates and manages registered
     models and model versions.
@@ -88,7 +88,7 @@ class ModelRegistryClient(object):
         Delete registered model.
         Backend raises exception if a registered model with given name does not exist.
 
-        :param name: Name of the registered model to update.
+        :param name: Name of the registered model to delete.
         """
         self.store.delete_registered_model(name)
 
@@ -131,7 +131,7 @@ class ModelRegistryClient(object):
 
     def get_registered_model(self, name):
         """
-        :param name: Name of the registered model to update.
+        :param name: Name of the registered model to get.
         :return: A single :py:class:`mlflow.entities.model_registry.RegisteredModel` object.
         """
         return self.store.get_registered_model(name)
@@ -141,7 +141,7 @@ class ModelRegistryClient(object):
         Latest version models for each requests stage. If no ``stages`` provided, returns the
         latest version for each stage.
 
-        :param name: Name of the registered model to update.
+        :param name: Name of the registered model from which to get the latest versions.
         :param stages: List of desired stages. If input list is None, return latest versions for
                        for 'Staging' and 'Production' stages.
         :return: List of :py:class:`mlflow.entities.model_registry.ModelVersion` objects.
@@ -248,7 +248,7 @@ class ModelRegistryClient(object):
         :param version: Registered model version.
         :param stage: New desired stage for this model version.
         :param archive_existing_versions: If this flag is set to ``True``, all existing model
-            versions in the stage will be automically moved to the "archived" stage. Only valid
+            versions in the stage will be automatically moved to the "archived" stage. Only valid
             when ``stage`` is ``"staging"`` or ``"production"`` otherwise an error will be raised.
 
         :return: A single :py:class:`mlflow.entities.model_registry.ModelVersion` object.
