@@ -16,17 +16,28 @@ MODEL_PATH = click.option(
     "run-id or local filesystem path without run-id.",
 )
 
+_model_uri_help_string = (
+    "URI to the model. A local path, a 'runs:/' URI, or a"
+    " remote storage URI (e.g., an 's3://' URI). For more information"
+    " about supported remote URIs for model artifacts, see"
+    " https://mlflow.org/docs/latest/tracking.html#artifact-stores"
+)
+
+MODEL_URI_BUILD_DOCKER = click.option(
+    "--model-uri",
+    "-m",
+    metavar="URI",
+    default=None,
+    required=False,
+    help="[Optional] " + _model_uri_help_string,
+)
+
 MODEL_URI = click.option(
     "--model-uri",
     "-m",
-    default=None,
     metavar="URI",
     required=True,
-    help="URI to the model. A local path, a 'runs:/' URI, or a"
-    " remote storage URI (e.g., an 's3://' URI). For more information"
-    " about supported remote URIs for model artifacts, see"
-    " https://mlflow.org/docs/latest/tracking.html"
-    "#artifact-stores",
+    help=_model_uri_help_string,
 )
 
 MLFLOW_HOME = click.option(
