@@ -2,20 +2,37 @@ import base64
 import json
 import operator
 import re
+import ast
 import shlex
 
 import sqlparse
-from sqlparse.sql import Identifier, Token, Comparison, Statement
+from sqlparse.sql import (
+    Identifier,
+    Token,
+    Comparison,
+    Statement,
+    Parenthesis,
+    IdentifierList,
+)
 from sqlparse.tokens import Token as TokenType
 
 from mlflow.entities import RunInfo
 from mlflow.exceptions import MlflowException
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
+from mlflow.store.db.db_types import MYSQL, MSSQL
 
 import math
 
 
-class SearchUtils(object):
+def _case_sensitive_match(string, pattern):
+    return re.match(pattern, string) is not None
+
+
+def _case_insensitive_match(string, pattern):
+    return re.match(pattern, string, flags=re.IGNORECASE) is not None
+
+
+class SearchUtils:
     LIKE_OPERATOR = "LIKE"
     ILIKE_OPERATOR = "ILIKE"
     ASC_OPERATOR = "asc"
@@ -25,10 +42,13 @@ class SearchUtils(object):
     VALID_PARAM_COMPARATORS = set(["!=", "=", LIKE_OPERATOR, ILIKE_OPERATOR])
     VALID_TAG_COMPARATORS = set(["!=", "=", LIKE_OPERATOR, ILIKE_OPERATOR])
     VALID_STRING_ATTRIBUTE_COMPARATORS = set(["!=", "=", LIKE_OPERATOR, ILIKE_OPERATOR])
+    VALID_NUMERIC_ATTRIBUTE_COMPARATORS = VALID_METRIC_COMPARATORS
+    NUMERIC_ATTRIBUTES = set(["start_time"])
     CASE_INSENSITIVE_STRING_COMPARISON_OPERATORS = set([LIKE_OPERATOR, ILIKE_OPERATOR])
     VALID_REGISTERED_MODEL_SEARCH_COMPARATORS = CASE_INSENSITIVE_STRING_COMPARISON_OPERATORS.union(
         {"="}
     )
+    VALID_MODEL_VERSIONS_SEARCH_COMPARATORS = set(["=", "IN"])
     VALID_SEARCH_ATTRIBUTE_KEYS = set(RunInfo.get_searchable_attributes())
     VALID_ORDER_BY_ATTRIBUTE_KEYS = set(RunInfo.get_orderable_attributes())
     _METRIC_IDENTIFIER = "metric"
@@ -48,6 +68,8 @@ class SearchUtils(object):
         + list(_ALTERNATE_ATTRIBUTE_IDENTIFIERS)
     )
     STRING_VALUE_TYPES = set([TokenType.Literal.String.Single])
+    DELIMITER_VALUE_TYPES = set([TokenType.Punctuation])
+    WHITESPACE_VALUE_TYPE = TokenType.Text.Whitespace
     NUMERIC_VALUE_TYPES = set([TokenType.Literal.Number.Integer, TokenType.Literal.Number.Float])
     # Registered Models Constants
     ORDER_BY_KEY_TIMESTAMP = "timestamp"
@@ -71,13 +93,43 @@ class SearchUtils(object):
         "!=": operator.ne,
         "<=": operator.le,
         "<": operator.lt,
-        "LIKE": re.match,
-        "ILIKE": re.match,
+        "LIKE": _case_sensitive_match,
+        "ILIKE": _case_insensitive_match,
     }
 
     @classmethod
-    def get_sql_filter_ops(cls, column, operator):
-        sql_filter_ops = {"LIKE": column.like, "ILIKE": column.ilike}
+    def get_sql_filter_ops(cls, column, operator, dialect):
+        import sqlalchemy as sa
+
+        col = f"{column.class_.__tablename__}.{column.key}"
+
+        # Use case-sensitive collation for MSSQL
+        if dialect == MSSQL:
+            column = column.collate("Japanese_Bushu_Kakusu_100_CS_AS_KS_WS")
+
+        # Use non-binary ahead of binary comparison for runtime performance
+        # Use non-binary ahead of binary comparison for runtime performance
+        def case_sensitive_mysql_eq(value):
+            return sa.text(f"({col} = :value AND BINARY {col} = :value)").bindparams(
+                sa.bindparam("value", value=value, unique=True)
+            )
+
+        def case_sensitive_mysql_ne(value):
+            return sa.text(f"({col} != :value OR BINARY {col} != :value)").bindparams(
+                sa.bindparam("value", value=value, unique=True)
+            )
+
+        def case_sensitive_mysql_like(value):
+            return sa.text(f"({col} LIKE :value AND BINARY {col} LIKE :value)").bindparams(
+                sa.bindparam("value", value=value, unique=True)
+            )
+
+        sql_filter_ops = {
+            "=": case_sensitive_mysql_eq if dialect == MYSQL else column.__eq__,
+            "!=": case_sensitive_mysql_ne if dialect == MYSQL else column.__ne__,
+            "LIKE": case_sensitive_mysql_like if dialect == MYSQL else column.like,
+            "ILIKE": column.ilike,
+        }
         return sql_filter_ops[operator]
 
     @classmethod
@@ -150,18 +202,18 @@ class SearchUtils(object):
         identifier = cls._valid_entity_type(entity_type)
         key = cls._trim_backticks(cls._strip_quotes(key))
         if identifier == cls._ATTRIBUTE_IDENTIFIER and key not in valid_attributes:
-            raise MlflowException(
+            raise MlflowException.invalid_parameter_value(
                 "Invalid attribute key '{}' specified. Valid keys "
-                " are '{}'".format(key, valid_attributes)
+                "are '{}'".format(key, valid_attributes)
             )
         return {"type": identifier, "key": key}
 
     @classmethod
-    def _get_value(cls, identifier_type, token):
+    def _get_value(cls, identifier_type, key, token):
         if identifier_type == cls._METRIC_IDENTIFIER:
             if token.ttype not in cls.NUMERIC_VALUE_TYPES:
                 raise MlflowException(
-                    "Expected numeric value type for metric. " "Found {}".format(token.value),
+                    "Expected numeric value type for metric. Found {}".format(token.value),
                     error_code=INVALID_PARAMETER_VALUE,
                 )
             return token.value
@@ -175,7 +227,15 @@ class SearchUtils(object):
                 error_code=INVALID_PARAMETER_VALUE,
             )
         elif identifier_type == cls._ATTRIBUTE_IDENTIFIER:
-            if token.ttype in cls.STRING_VALUE_TYPES or isinstance(token, Identifier):
+            if key in cls.NUMERIC_ATTRIBUTES:
+                if token.ttype not in cls.NUMERIC_VALUE_TYPES:
+                    raise MlflowException(
+                        "Expected numeric value type for numeric attribute: {}. "
+                        "Found {}".format(key, token.value),
+                        error_code=INVALID_PARAMETER_VALUE,
+                    )
+                return token.value
+            elif token.ttype in cls.STRING_VALUE_TYPES or isinstance(token, Identifier):
                 return cls._strip_quotes(token.value, expect_quoted_value=True)
             else:
                 raise MlflowException(
@@ -223,11 +283,11 @@ class SearchUtils(object):
         cls._validate_comparison(stripped_comparison)
         comp = cls._get_identifier(stripped_comparison[0].value, cls.VALID_SEARCH_ATTRIBUTE_KEYS)
         comp["comparator"] = stripped_comparison[1].value
-        comp["value"] = cls._get_value(comp.get("type"), stripped_comparison[2])
+        comp["value"] = cls._get_value(comp.get("type"), comp.get("key"), stripped_comparison[2])
         return comp
 
     @classmethod
-    def _invalid_statement_token(cls, token):
+    def _invalid_statement_token_search_runs(cls, token):
         if isinstance(token, Comparison):
             return False
         elif token.is_whitespace:
@@ -240,7 +300,7 @@ class SearchUtils(object):
     @classmethod
     def _process_statement(cls, statement):
         # check validity
-        invalids = list(filter(cls._invalid_statement_token, statement.tokens))
+        invalids = list(filter(cls._invalid_statement_token_search_runs, statement.tokens))
         if len(invalids) > 0:
             invalid_clauses = ", ".join("'%s'" % token for token in invalids)
             raise MlflowException(
@@ -270,7 +330,7 @@ class SearchUtils(object):
                 "Provide AND-ed expression list." % filter_string,
                 error_code=INVALID_PARAMETER_VALUE,
             )
-        return SearchUtils._process_statement(parsed[0])
+        return cls._process_statement(parsed[0])
 
     @classmethod
     def is_metric(cls, key_type, comparator):
@@ -308,8 +368,8 @@ class SearchUtils(object):
         return False
 
     @classmethod
-    def is_attribute(cls, key_type, comparator):
-        if key_type == cls._ATTRIBUTE_IDENTIFIER:
+    def is_string_attribute(cls, key_type, key_name, comparator):
+        if key_type == cls._ATTRIBUTE_IDENTIFIER and key_name not in cls.NUMERIC_ATTRIBUTES:
             if comparator not in cls.VALID_STRING_ATTRIBUTE_COMPARATORS:
                 raise MlflowException(
                     "Invalid comparator '{}' not one of "
@@ -318,6 +378,25 @@ class SearchUtils(object):
             return True
         return False
 
+    @classmethod
+    def is_numeric_attribute(cls, key_type, key_name, comparator):
+        if key_type == cls._ATTRIBUTE_IDENTIFIER and key_name in cls.NUMERIC_ATTRIBUTES:
+            if comparator not in cls.VALID_NUMERIC_ATTRIBUTE_COMPARATORS:
+                raise MlflowException(
+                    "Invalid comparator '{}' not one of "
+                    "'{}".format(comparator, cls.VALID_STRING_ATTRIBUTE_COMPARATORS)
+                )
+            return True
+        return False
+
+    @classmethod
+    def _convert_like_pattern_to_regex(cls, pattern):
+        if not pattern.startswith("%"):
+            pattern = "^" + pattern
+        if not pattern.endswith("%"):
+            pattern = pattern + "$"
+        return pattern.replace("_", ".").replace("%", ".*")
+
     @classmethod
     def _does_run_match_clause(cls, run, sed):
         key_type = sed.get("type")
@@ -332,8 +411,11 @@ class SearchUtils(object):
             lhs = run.data.params.get(key, None)
         elif cls.is_tag(key_type, comparator):
             lhs = run.data.tags.get(key, None)
-        elif cls.is_attribute(key_type, comparator):
+        elif cls.is_string_attribute(key_type, key, comparator):
             lhs = getattr(run.info, key)
+        elif cls.is_numeric_attribute(key_type, key, comparator):
+            lhs = getattr(run.info, key)
+            value = int(value)
         else:
             raise MlflowException(
                 "Invalid search expression type '%s'" % key_type, error_code=INVALID_PARAMETER_VALUE
@@ -342,18 +424,9 @@ class SearchUtils(object):
             return False
 
         if comparator in cls.CASE_INSENSITIVE_STRING_COMPARISON_OPERATORS:
-            # Change value from sql syntax to regex syntax
-            if comparator == "ILIKE":
-                value = value.lower()
-                lhs = lhs.lower()
-            if not value.startswith("%"):
-                value = "^" + value
-            if not value.endswith("%"):
-                value = value + "$"
-            value = value.replace("_", ".").replace("%", ".*")
-            return cls.filter_ops.get(comparator)(value, lhs)
+            value = cls._convert_like_pattern_to_regex(value)
 
-        elif comparator in cls.filter_ops.keys():
+        if comparator in cls.filter_ops.keys():
             return cls.filter_ops.get(comparator)(lhs, value)
         else:
             return False
@@ -366,7 +439,7 @@ class SearchUtils(object):
         parsed = cls.parse_search_filter(filter_string)
 
         def run_matches(run):
-            return all([cls._does_run_match_clause(run, s) for s in parsed])
+            return all(cls._does_run_match_clause(run, s) for s in parsed)
 
         return [run for run in runs if run_matches(run)]
 
@@ -393,7 +466,7 @@ class SearchUtils(object):
             token_value = cls.ORDER_BY_KEY_TIMESTAMP
         elif (
             statement.tokens[0].match(ttype=TokenType.Keyword, values=[cls.ORDER_BY_KEY_TIMESTAMP])
-            and all([token.is_whitespace for token in statement.tokens[1:-1]])
+            and all(token.is_whitespace for token in statement.tokens[1:-1])
             and statement.tokens[-1].ttype == TokenType.Keyword.Order
         ):
             token_value = cls.ORDER_BY_KEY_TIMESTAMP + " " + statement.tokens[-1].value
@@ -461,12 +534,18 @@ class SearchUtils(object):
             )
 
         # Return a key such that None values are always at the end.
-        is_null_or_nan = sort_value is None or (
-            isinstance(sort_value, float) and math.isnan(sort_value)
-        )
-        if ascending:
-            return (is_null_or_nan, sort_value)
-        return (not is_null_or_nan, sort_value)
+        is_none = sort_value is None
+        is_nan = isinstance(sort_value, float) and math.isnan(sort_value)
+        fill_value = (1 if ascending else -1) * math.inf
+
+        if is_none:
+            sort_value = fill_value
+        elif is_nan:
+            sort_value = -fill_value
+
+        is_none_or_nan = is_none or is_nan
+
+        return (is_none_or_nan, sort_value) if ascending else (not is_none_or_nan, sort_value)
 
     @classmethod
     def sort(cls, runs, order_by_list):
@@ -559,75 +638,292 @@ class SearchUtils(object):
     VALID_SEARCH_KEYS_FOR_REGISTERED_MODELS = set(["name"])
 
     @classmethod
-    def _get_comparison_for_model_registry(cls, comparison, valid_search_keys):
+    def _check_valid_identifier_list(cls, value_token):
+        if len(value_token._groupable_tokens) == 0:
+            raise MlflowException(
+                "While parsing a list in the query,"
+                " expected a non-empty list of string values, but got empty list",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+        elif not isinstance(value_token._groupable_tokens[0], IdentifierList):
+            raise MlflowException(
+                "While parsing a list in the query,"
+                " expected a non-empty list of string values, but got ill-formed list.",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+        elif not all(
+            map(
+                lambda token: token.ttype
+                in {*cls.STRING_VALUE_TYPES, *cls.DELIMITER_VALUE_TYPES, cls.WHITESPACE_VALUE_TYPE},
+                value_token._groupable_tokens[0].tokens,
+            )
+        ):
+            raise MlflowException(
+                "While parsing a list in the query, expected string value, punctuation, "
+                "or whitespace, but got different type in list: {value_token}".format(
+                    value_token=value_token
+                ),
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+
+    @classmethod
+    def _parse_list_from_sql_token(cls, token):
+        try:
+            return ast.literal_eval(token.value)
+        except SyntaxError:
+            raise MlflowException(
+                "While parsing a list in the query,"
+                " expected a non-empty list of string values, but got ill-formed list.",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+
+
+class SearchExperimentsUtils(SearchUtils):
+    VALID_SEARCH_ATTRIBUTE_KEYS = ("name",)
+    VALID_ORDER_BY_ATTRIBUTE_KEYS = ("name", "experiment_id")
+
+    @classmethod
+    def _invalid_statement_token_search_experiments(cls, token):
+        if isinstance(token, (Comparison, Identifier, Parenthesis)):
+            return False
+        elif token.is_whitespace:
+            return False
+        elif token.match(ttype=TokenType.Keyword, values=["AND"]):
+            return False
+        else:
+            return True
+
+    @classmethod
+    def _process_statement(cls, statement):
+        invalids = list(filter(cls._invalid_statement_token_search_experiments, statement.tokens))
+        if len(invalids) > 0:
+            invalid_clauses = ", ".join(map(str, invalids))
+            raise MlflowException.invalid_parameter_value(
+                "Invalid clause(s) in filter string: %s" % invalid_clauses
+            )
+        return [cls._get_comparison(t) for t in statement.tokens if isinstance(t, Comparison)]
+
+    @classmethod
+    def _get_identifier(cls, identifier, valid_attributes):
+        tokens = identifier.split(".", maxsplit=1)
+        if len(tokens) == 1:
+            key = tokens[0]
+            identifier = cls._ATTRIBUTE_IDENTIFIER
+        else:
+            entity_type, key = tokens
+            valid_entity_types = ("attribute", "tag", "tags")
+            if entity_type not in valid_entity_types:
+                raise MlflowException.invalid_parameter_value(
+                    f"Invalid entity type '{entity_type}'. "
+                    f"Valid entity types are {valid_entity_types}"
+                )
+            identifier = cls._valid_entity_type(entity_type)
+
+        key = cls._trim_backticks(cls._strip_quotes(key))
+        if identifier == cls._ATTRIBUTE_IDENTIFIER and key not in valid_attributes:
+            raise MlflowException.invalid_parameter_value(
+                "Invalid attribute key '{}' specified. Valid keys "
+                "are '{}'".format(key, valid_attributes)
+            )
+        return {"type": identifier, "key": key}
+
+    @classmethod
+    def _get_comparison(cls, comparison):
         stripped_comparison = [token for token in comparison.tokens if not token.is_whitespace]
         cls._validate_comparison(stripped_comparison)
-        key = stripped_comparison[0].value
-        if key not in valid_search_keys:
-            raise MlflowException(
-                "Invalid attribute key '{}' specified. Valid keys "
-                " are '{}'".format(key, valid_search_keys),
-                error_code=INVALID_PARAMETER_VALUE,
-            )
-        value_token = stripped_comparison[2]
-        if value_token.ttype not in cls.STRING_VALUE_TYPES:
-            raise MlflowException(
-                "Expected a quoted string value for attributes. "
-                "Got value {value}".format(value=value_token.value),
-                error_code=INVALID_PARAMETER_VALUE,
-            )
-        comp = {
-            "key": key,
-            "comparator": stripped_comparison[1].value,
-            "value": cls._strip_quotes(value_token.value, expect_quoted_value=True),
-        }
+        left, comparator, right = stripped_comparison
+        comp = cls._get_identifier(left.value, cls.VALID_SEARCH_ATTRIBUTE_KEYS)
+        comp["comparator"] = comparator.value
+        comp["value"] = cls._get_value(comp.get("type"), comp.get("key"), right)
         return comp
 
     @classmethod
-    def _parse_filter_for_model_registry(cls, filter_string, valid_search_keys):
-        if not filter_string or filter_string == "":
-            return []
-        expected = "Expected search filter with single comparison operator. e.g. name='myModelName'"
-        try:
-            parsed = sqlparse.parse(filter_string)
-        except Exception:
+    def parse_order_by_for_search_experiments(cls, order_by):
+        token_value, is_ascending = cls._parse_order_by_string(order_by)
+        identifier = cls._get_identifier(token_value.strip(), cls.VALID_ORDER_BY_ATTRIBUTE_KEYS)
+        return identifier["type"], identifier["key"], is_ascending
+
+    @classmethod
+    def is_attribute(cls, key_type, comparator):
+        if key_type == cls._ATTRIBUTE_IDENTIFIER:
+            if comparator not in cls.VALID_STRING_ATTRIBUTE_COMPARATORS:
+                raise MlflowException(
+                    "Invalid comparator '{}' not one of "
+                    "'{}".format(comparator, cls.VALID_STRING_ATTRIBUTE_COMPARATORS)
+                )
+            return True
+        return False
+
+    @classmethod
+    def _does_experiment_match_clause(cls, experiment, sed):  # pylint: disable=arguments-renamed
+        key_type = sed.get("type")
+        key = sed.get("key")
+        value = sed.get("value")
+        comparator = sed.get("comparator").upper()
+
+        if cls.is_attribute(key_type, comparator):
+            lhs = getattr(experiment, key)
+        elif cls.is_tag(key_type, comparator):
+            if key not in experiment.tags:
+                return False
+            lhs = experiment.tags.get(key, None)
+            if lhs is None:
+                return experiment
+        else:
             raise MlflowException(
-                "Error while parsing filter '%s'. %s" % (filter_string, expected),
-                error_code=INVALID_PARAMETER_VALUE,
+                "Invalid search expression type '%s'" % key_type, error_code=INVALID_PARAMETER_VALUE
             )
-        if len(parsed) == 0 or not isinstance(parsed[0], Statement):
-            raise MlflowException(
-                "Invalid filter '%s'. Could not be parsed. %s" % (filter_string, expected),
-                error_code=INVALID_PARAMETER_VALUE,
-            )
-        elif len(parsed) > 1:
-            raise MlflowException(
-                "Search filter '%s' contains multiple expressions. "
-                "%s " % (filter_string, expected),
-                error_code=INVALID_PARAMETER_VALUE,
-            )
-        statement = parsed[0]
-        invalids = list(filter(cls._invalid_statement_token, statement.tokens))
+
+        if comparator in cls.CASE_INSENSITIVE_STRING_COMPARISON_OPERATORS:
+            value = cls._convert_like_pattern_to_regex(value)
+
+        if comparator in cls.filter_ops.keys():
+            return cls.filter_ops.get(comparator)(lhs, value)
+        else:
+            return False
+
+    @classmethod
+    def filter(cls, experiments, filter_string):  # pylint: disable=arguments-renamed
+        if not filter_string:
+            return experiments
+        parsed = cls.parse_search_filter(filter_string)
+
+        def experiment_matches(experiment):
+            return all(cls._does_experiment_match_clause(experiment, s) for s in parsed)
+
+        return list(filter(experiment_matches, experiments))
+
+    @classmethod
+    def _get_sort_key(cls, order_by_list):
+        order_by = []
+        parsed_order_by = map(cls.parse_order_by_for_search_experiments, order_by_list or [])
+        for type_, key, ascending in parsed_order_by:
+            if type_ == "attribute":
+                order_by.append((key, ascending))
+            else:
+                raise MlflowException.invalid_parameter_value(f"Invalid order_by entity: {type_}")
+
+        # Add a tie-breaker
+        if not any(key == "experiment_id" for key, _ in order_by):
+            order_by.append(("experiment_id", False))
+
+        # https://stackoverflow.com/a/56842689
+        class _Reversor:
+            def __init__(self, obj):
+                self.obj = obj
+
+            # Only need < and == are needed for use as a key parameter in the sorted function
+            def __eq__(self, other):
+                return other.obj == self.obj
+
+            def __lt__(self, other):
+                return other.obj < self.obj
+
+        def _apply_reversor(experiment, key, ascending):
+            attr = getattr(experiment, key)
+            return attr if ascending else _Reversor(attr)
+
+        return lambda experiment: tuple(
+            _apply_reversor(experiment, k, asc) for (k, asc) in order_by
+        )
+
+    @classmethod
+    def sort(cls, experiments, order_by_list):  # pylint: disable=arguments-renamed
+        return sorted(experiments, key=cls._get_sort_key(order_by_list))
+
+
+class SearchModelUtils(SearchUtils):
+    @classmethod
+    def _process_statement(cls, statement):
+        invalids = list(
+            filter(cls._invalid_statement_token_search_model_registry, statement.tokens)
+        )
         if len(invalids) > 0:
-            invalid_clauses = ", ".join("'%s'" % token for token in invalids)
+            invalid_clauses = ", ".join(map(str, invalids))
+            raise MlflowException.invalid_parameter_value(
+                "Invalid clause(s) in filter string: %s" % invalid_clauses
+            )
+        return [cls._get_comparison(t) for t in statement.tokens if isinstance(t, Comparison)]
+
+    @classmethod
+    def _get_model_search_identifier(cls, identifier):
+        tokens = identifier.split(".", maxsplit=1)
+        if len(tokens) == 1:
+            key = tokens[0]
+            identifier = cls._ATTRIBUTE_IDENTIFIER
+        else:
+            entity_type, key = tokens
+            valid_entity_types = ("attribute", "tag", "tags")
+            if entity_type not in valid_entity_types:
+                raise MlflowException.invalid_parameter_value(
+                    f"Invalid entity type '{entity_type}'. "
+                    f"Valid entity types are {valid_entity_types}"
+                )
+            identifier = (
+                cls._TAG_IDENTIFIER if entity_type in ("tag", "tags") else cls._ATTRIBUTE_IDENTIFIER
+            )
+
+        key = cls._trim_backticks(cls._strip_quotes(key))
+        return {"type": identifier, "key": key}
+
+    @classmethod
+    def _get_comparison(cls, comparison):
+        stripped_comparison = [token for token in comparison.tokens if not token.is_whitespace]
+        cls._validate_comparison(stripped_comparison)
+        left, comparator, right = stripped_comparison
+        comp = cls._get_model_search_identifier(left.value)
+        comp["comparator"] = comparator.value.upper()
+        comp["value"] = cls._get_value(comp.get("type"), comp.get("key"), right)
+        return comp
+
+    @classmethod
+    def _get_value(cls, identifier_type, key, token):
+        if identifier_type == cls._TAG_IDENTIFIER:
+            if token.ttype in cls.STRING_VALUE_TYPES or isinstance(token, Identifier):
+                return cls._strip_quotes(token.value, expect_quoted_value=True)
             raise MlflowException(
-                "Invalid clause(s) in filter string: %s. " "%s" % (invalid_clauses, expected),
+                "Expected a quoted string value for "
+                "{identifier_type} (e.g. 'my-value'). Got value "
+                "{value}".format(identifier_type=identifier_type, value=token.value),
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+        elif identifier_type == cls._ATTRIBUTE_IDENTIFIER:
+            if token.ttype in cls.STRING_VALUE_TYPES or isinstance(token, Identifier):
+                return cls._strip_quotes(token.value, expect_quoted_value=True)
+            elif isinstance(token, Parenthesis):
+                cls._check_valid_identifier_list(token)
+                if key != "run_id":
+                    raise MlflowException(
+                        "Only run_id attribute support compare with a list of quoted string "
+                        "values.",
+                        error_code=INVALID_PARAMETER_VALUE,
+                    )
+                run_id_list = cls._parse_list_from_sql_token(token)
+                # Because MySQL IN clause is case-insensitive, but all run_ids only contain lower
+                # case letters, so that we filter out run_ids containing upper case letters here.
+                run_id_list = [run_id for run_id in run_id_list if run_id.islower()]
+                return run_id_list
+            else:
+                raise MlflowException(
+                    "Expected a quoted string value or a list of quoted string values for "
+                    "attributes. Got value {value}".format(value=token.value),
+                    error_code=INVALID_PARAMETER_VALUE,
+                )
+        else:
+            # Expected to be either "param" or "metric".
+            raise MlflowException(
+                "Invalid identifier type. Expected one of "
+                "{}.".format([cls._ATTRIBUTE_IDENTIFIER, cls._TAG_IDENTIFIER]),
                 error_code=INVALID_PARAMETER_VALUE,
             )
-        return [
-            cls._get_comparison_for_model_registry(si, valid_search_keys)
-            for si in statement.tokens
-            if isinstance(si, Comparison)
-        ]
 
     @classmethod
-    def parse_filter_for_model_versions(cls, filter_string):
-        return cls._parse_filter_for_model_registry(
-            filter_string, cls.VALID_SEARCH_KEYS_FOR_MODEL_VERSIONS
-        )
-
-    @classmethod
-    def parse_filter_for_registered_models(cls, filter_string):
-        return cls._parse_filter_for_model_registry(
-            filter_string, cls.VALID_SEARCH_KEYS_FOR_REGISTERED_MODELS
-        )
+    def _invalid_statement_token_search_model_registry(cls, token):
+        if isinstance(token, (Comparison, Identifier, Parenthesis)):
+            return False
+        elif token.is_whitespace:
+            return False
+        elif token.match(ttype=TokenType.Keyword, values=["AND", "IN"]):
+            return False
+        else:
+            return True
